@@ -13,28 +13,32 @@ use overload ( '""'  => \&as_string );
 use Games::Domino::Tile;
 use Games::Domino::Player;
 
+select(STDOUT);
+$|=1;
+
+$SIG{'INT'} = sub { print {*STDOUT} "\n\nCaught Interrupt (^C), Aborting the game.\n"; exit(1); };
+
 =head1 NAME
 
 Games::Domino - Interface to the Domino game.
 
 =head1 VERSION
 
-Version 0.02
+Version 0.03
 
 =cut
 
-our $VERSION = '0.02';
+our $VERSION = '0.03';
 
-has 'stock'     => (is => 'rw', isa => 'ArrayRef[Games::Domino::Tile]');
-has 'board'     => (is => 'rw', isa => 'ArrayRef[Games::Domino::Tile]');
-has 'human'     => (is => 'rw', isa => 'Games::Domino::Player');
-has 'computer'  => (is => 'rw', isa => 'Games::Domino::Player');
-has 'current'   => (is => 'rw', isa => 'Games::Domino::Player');
-has 'open_ends' => (is => 'rw', isa => 'ArrayRef[ZeroToSix]');
-has 'board_l'   => (is => 'rw', isa => 'ZeroToSix');
-has 'board_r'   => (is => 'rw', isa => 'ZeroToSix');
-has 'cheat'     => (is => 'ro', isa => 'ZeroOrOne', default => 0);
-has 'debug'     => (is => 'rw', isa => 'ZeroOrOne', default => 0);
+has 'stock'    => (is => 'rw', isa => 'ArrayRef[Games::Domino::Tile]');
+has 'board'    => (is => 'rw', isa => 'ArrayRef[Games::Domino::Tile]');
+has 'human'    => (is => 'rw', isa => 'Games::Domino::Player');
+has 'computer' => (is => 'rw', isa => 'Games::Domino::Player');
+has 'current'  => (is => 'rw', isa => 'Games::Domino::Player');
+has 'board_l'  => (is => 'rw', isa => 'ZeroToSix');
+has 'board_r'  => (is => 'rw', isa => 'ZeroToSix');
+has 'cheat'    => (is => 'ro', isa => 'ZeroOrOne', default => 0);
+has 'debug'    => (is => 'rw', isa => 'ZeroOrOne', default => 0);
 
 =head1 DESCRIPTION
 
@@ -69,30 +73,24 @@ sub BUILD
 
     $self->{stock} = $self->_prepare();
     $self->{human} = Games::Domino::Player->new({ name => 'H', show => 1 });
-    if ($self->cheat)
-    {
+
+    if ($self->cheat) {
         $self->{computer} = Games::Domino::Player->new({ name => 'C', show => 1 });
-    }
-    else
-    {
+    } else {
         $self->{computer} = Games::Domino::Player->new({ name => 'C' });
     }
 
     $self->{human}->save(shift @{$self->{stock}})    for (1..7);
     $self->{computer}->save(shift @{$self->{stock}}) for (1..7);
-    $self->{current} = $self->{human};
+    $self->{current} = $self->{computer};
 
-    if ($self->debug)
-    {
-        print {*STDOUT} "H: " . $self->human    . "\n";
-        print {*STDOUT} "C: " . $self->computer . "\n";
-        print {*STDOUT} "\n\n";
-    }
+    $self->_instructions;
+    $self->show if $self->debug;
 }
 
 =head1 METHODS
 
-=head2 draw()
+=head2 play()
 
 Pick a tile from the  current  player. If  no matching tile found then picks it from the stock
 until it found one or the stock has only 2 tiles left at that time the game is over.
@@ -101,100 +99,46 @@ until it found one or the stock has only 2 tiles left at that time the game is o
     use Games::Domino;
 
     my $game = Games::Domino->new();
-    my $tile = $game->draw();
+    $game->play;
 
 =cut
 
-sub draw
-{
+sub play {
     my $self = shift;
-    my $tile = $self->_next()->pick($self->open_ends);
-    return $tile if defined $tile;
 
-    while (scalar(@{$self->{stock}}) > 2)
-    {
-        $tile = $self->_pick();
-        $self->{current}->save($tile);
-        $tile = $self->{current}->pick($self->open_ends);
+    my $player = $self->{current};
+    if ($player->name eq 'C') {
+        my $tile = $player->pick($self->board_l, $self->board_r);
+        if (defined $tile) {
+            print {*STDOUT} "[C] [P]: $tile [S]\n" if $self->debug;
+            $self->_save($tile);
+            return $tile;
+        }
+
+        $tile = $self->_play($player->name);
         return $tile if defined $tile;
+
+    } else {
+        my $index;
+        do {
+            print {*STDOUT} "Pick your tile [" . $player->_available_indexes . "][B]? ";
+            $index = <STDIN>;
+            chomp($index) if defined $index;
+
+            if (defined($index) && ($index =~ /B/i)) {
+                my $tile = $self->_play($player->name);
+                return $tile if defined $tile;
+            }
+        } until (defined($index) && $player->_validate_index($index) && $player->_validate_tile($index, $self->board_l, $self->board_r));
+
+        my $tile = $player->_tile($index);
+        print {*STDOUT} "[H] [P]: $tile [S]\n" if $self->debug;
+        splice(@{$player->{bank}}, $index-1, 1);
+        $self->_save($tile);
+        return $tile;
     }
 }
 
-=head2 save()
-
-Saves the given tile and arrange it properly on the board. Also capture the open ends for next
-move.
-
-    use strict; use warnings;
-    use Games::Domino;
-
-    my $game = Games::Domino->new();
-    my $tile = $game->draw();
-    $game->save($tile);
-
-=cut
-
-sub save
-{
-    my $self = shift;
-    my $tile = shift;
-
-    if (!defined($self->{board}) || (scalar(@{$self->{board}}) == 0))
-    {
-        push @{$self->{board}}, $tile;
-        $self->{board_l} = $tile->left;
-        $self->{board_r} = $tile->right;
-        $self->_open_ends([$self->{board_l}, $self->{board_r}]);
-        $self->_next;
-        return;
-    }
-
-    if ($self->{board_r} == $tile->left)
-    {
-        push @{$self->{board}}, $tile;
-        $self->{board_r} = $tile->right;
-        $self->_open_ends([$self->{board_l}, $self->{board_r}]);
-        $self->_next;
-        return;
-    }
-    elsif ($self->{board_r} == $tile->right)
-    {
-        my $L = $tile->left;
-        my $R = $tile->right;
-        $tile->right($L);
-        $tile->left($R);
-        push @{$self->{board}}, $tile;
-        $self->{board_r} = $L;
-        $self->_open_ends([$self->{board_l}, $self->{board_r}]);
-        $self->_next;
-        return;
-    }
-
-    if ($self->{board_l} == $tile->left)
-    {
-        my $L = $tile->left;
-        my $R = $tile->right;
-        $tile->right($L);
-        $tile->left($R);
-        unshift @{$self->{board}}, $tile;
-        $self->{board_l} = $R;
-        $self->_open_ends([$self->{board_l}, $self->{board_r}]);
-        $self->_next;
-        return;
-
-    }
-    elsif ($self->{board_l} == $tile->right)
-    {
-        unshift @{$self->{board}}, $tile;
-        $self->{board_l} = $tile->left;
-        $self->_open_ends([$self->{board_l}, $self->{board_r}]);
-        $self->_next;
-        return;
-    }
-
-    return;
-}
-
 =head2 is_over()
 
 Returns 1 or 0 depending whether the game is over or not. The game can be declared over in the
@@ -212,16 +156,11 @@ following circumstances:
     use Games::Domino;
 
     my $game = Games::Domino->new();
-    do
-    {
-        my $tile = $game->draw();
-        $game->save($tile) if defined $tile;
-    } until ($game->is_over());
+    do { $game->play; } until $game->is_over;
 
 =cut
 
-sub is_over
-{
+sub is_over {
     my $self = shift;
     return 1 if ((scalar(@{$self->{stock}}) == 2)
                  ||
@@ -231,7 +170,7 @@ sub is_over
     return 0;
 }
 
-=head2 get_winner()
+=head2 result()
 
 Declares who is the winner against whom and by how much margin.
 
@@ -239,61 +178,50 @@ Declares who is the winner against whom and by how much margin.
     use Games::Domino;
 
     my $game = Games::Domino->new();
-    do
-    {
-        my $tile = $game->draw();
-        $game->save($tile) if defined $tile;
-    } until ($game->is_over());
-
-    print "WINNER: " . $game->get_winner() . "\n";
+    do { $game->play; } until $game->is_over;
+    $game->result;
 
 =cut
 
-sub get_winner
-{
+sub result {
     my $self = shift;
+
+    print {*STDOUT} "STOCK : $self\n";
     my $h = $self->{human}->value();
     my $c = $self->{computer}->value();
-    if ($h > $c)
-    {
-        return "C [$c] against H [$h]";
-    }
-    else
-    {
-        return "H [$h] against C [$c]";
+    if ($h == $c) {
+        if (scalar(@{$self->{computer}->{bank}}) < scalar(@{$self->{human}->{bank}})) {
+            print {*STDOUT} "WINNER: [C] [$c] against [H] [$h]\n";
+        } else {
+            print {*STDOUT} "WINNER: [H] [$h] against [C] [$c]\n";
+        }
+    } elsif ($h > $c) {
+        print {*STDOUT} "WINNER: [C] [$c] against [H] [$h]\n";
+    } else {
+        print {*STDOUT} "WINNER: [H] [$h] against [C] [$c]\n";
     }
+    $self->_line;
 }
 
-=head2 get_board()
+=head2 show()
 
-Returns all the tiles that were arranged during the game.
+Print the current tiles of Computer, Human and matched one.
 
     use strict; use warnings;
     use Games::Domino;
 
     my $game = Games::Domino->new();
-    do
-    {
-        my $tile = $game->draw();
-        $game->save($tile) if defined $tile;
-    } until ($game->is_over());
-
-    print "BOARD :" . $game->get_board() . "\n";
+    do { $game->play; $game->show; } until $game->is_over;
 
 =cut
 
-sub get_board
-{
+sub show {
     my $self = shift;
-
-    my $board = '';
-    foreach (@{$self->{board}})
-    {
-        $board .= sprintf("%s == ", $_);
-    }
-    $board =~ s/[\=]+\s?$//;
-    $board =~ s/\s+$//;
-    return $board;
+    $self->_line;
+    print {*STDOUT} "[C]: " . $self->computer . "\n";
+    print {*STDOUT} "[H]: " . $self->human    . "\n";
+    print {*STDOUT} "[G]: " . $self->_board() . "\n";
+    $self->_line;
 }
 
 =head2 as_string()
@@ -304,50 +232,167 @@ Returns all the unused tiles remained in the bank.
     use Games::Domino;
 
     my $game = Games::Domino->new();
-    do
-    {
-        my $tile = $game->draw();
-        $game->save($tile) if defined $tile;
-    } until ($game->is_over());
-
-    print "STOCK : $game\n\n";
+    do { $game->play; } until $game->is_over;
+    print "GAME : $game\n\n";
 
 =cut
 
-sub as_string
-{
+sub as_string {
     my $self = shift;
     return '[EMPTY]' unless scalar(@{$self->{stock}});
 
-    my $stock = '';
-    foreach (@{$self->{stock}})
-    {
-        $stock .= sprintf("%s == ", $_);
+    my $domino = '';
+    foreach (@{$self->{stock}}) {
+        $domino .= sprintf("%s==", $_);
     }
-    $stock =~ s/[\=]+\s?$//;
-    $stock =~ s/\s+$//;
-    return $stock;
+    $domino =~ s/[\=]+\s?$//;
+    $domino =~ s/\s+$//;
+    return $domino;
 }
 
-sub _pick
-{
+sub _instructions {
+    my $self = shift;
+    my $help = qq {
+   _____                               _____                  _
+  / ____|                          _ _|  __ \\                (_)
+ | |  __  __ _ _ __ ___   ___  ___(_|_) |  | | ___  _ __ ___  _ _ __   ___
+ | | |_ |/ _` | '_ ` _ \\ / _ \\/ __|   | |  | |/ _ \\| '_ ` _ \\| | '_ \\ / _ \\
+ | |__| | (_| | | | | | |  __/\\__ \\_ _| |__| | (_) | | | | | | | | | | (_) \|
+  \\_____|\\__,_|_| |_| |_|\\___||___(_|_)_____/ \\___/|_| |_| |_|_|_| |_|\\___/
+
+Tiles are numbered left to right starting with 1. Symbols used in this game are:
+    [C]: Code for the computer player
+    [H]: Code for the human player
+    [P]: Personal tile
+    [B]: Tile picked from the bank
+    [S]: Successfully found the matching tile
+    [F]: Unable to find the matching tile
+    [G]: All matched tiles so far
+
+Example:
+
+[C] [P]: [5 | 6] [S]
+Computer picked the tile [5 | 6] from his own collection and successfully found the matching on board.
+
+[H] [P]: [6 | 6] [S]
+Human picked the tile [6 | 6] from his own collection and successfully found the matching on board.
+
+[C] [B]: [2 | 6] [S]
+Computer randomly picked the tile [2 | 6] from the bank and successfully found the matching on board.
+
+[C] [B]: [3 | 4] [F]
+Computer randomly picked the tile [3 | 4] from the bank and but failed to find the matching on board.
+
+[H] [B]: [2 | 2] [S]
+Human randomly picked the tile [2 | 2] from the bank and successfully found the matching on board.
+
+[H] [B]: [3 | 6] [F]
+Human randomly picked the tile [3 | 6] from the bank and but failed to find the matching on board.
+};
+    $self->_line;
+    print {*STDOUT} $help,"\n";
+    $self->_line;
+}
+
+sub _play {
+    my $self   = shift;
+    my $player = $self->{current};
+    my $name   = $player->name;
+    while (scalar(@{$self->{stock}}) > 2) {
+        my $_tile = $self->_pick();
+        $player->save($_tile);
+        my $tile = $player->pick($self->board_l, $self->board_r);
+        if (defined $tile) {
+            print {*STDOUT} "[$name] [B]: $tile [S]\n" if $self->debug;
+            $self->_save($tile);
+            return $tile;
+        } else {
+            print {*STDOUT} "[$name] [B]: $_tile [F]\n" if $self->debug;
+        }
+    }
+}
+
+sub _save {
+    my $self = shift;
+    my $tile = shift;
+
+    if (!defined($self->{board}) || (scalar(@{$self->{board}}) == 0)) {
+        push @{$self->{board}}, $tile;
+        $self->{board_l} = $tile->left;
+        $self->{board_r} = $tile->right;
+        $self->_next;
+        return;
+    }
+
+    if ($self->{board_r} == $tile->left) {
+        push @{$self->{board}}, $tile;
+        $self->{board_r} = $tile->right;
+        $self->_next;
+        return;
+
+    } elsif ($self->{board_r} == $tile->right) {
+        my $L = $tile->left;
+        my $R = $tile->right;
+        $tile->right($L);
+        $tile->left($R);
+        push @{$self->{board}}, $tile;
+        $self->{board_r} = $L;
+        $self->_next;
+        return;
+    }
+
+    if ($self->{board_l} == $tile->left) {
+        my $L = $tile->left;
+        my $R = $tile->right;
+        $tile->right($L);
+        $tile->left($R);
+        unshift @{$self->{board}}, $tile;
+        $self->{board_l} = $R;
+        $self->_next;
+        return;
+
+    } elsif ($self->{board_l} == $tile->right) {
+        unshift @{$self->{board}}, $tile;
+        $self->{board_l} = $tile->left;
+        $self->_next;
+        return;
+    }
+
+    return;
+}
+
+sub _board {
+    my $self = shift;
+
+    my $board = '';
+    foreach (@{$self->{board}}) {
+        $board .= sprintf("%s==", $_);
+    }
+    $board =~ s/[\=]+\s?$//;
+    $board =~ s/\s+$//;
+    return $board;
+}
+
+sub _line {
+    my $self = shift;
+    print {*STDOUT} "="x76,"\n";
+}
+
+sub _pick {
     my $self = shift;
     return shift @{$self->{stock}};
 }
 
-sub _prepare
-{
+sub _prepare {
     my $self  = shift;
 
     my $tiles = [];
     my $tile  = Games::Domino::Tile->new({ left => 0, right => 0, double => 1 });
     push @$tiles, $tile;
-    foreach my $R (1..6)
-    {
+    foreach my $R (1..6) {
         my $L = 0;
         my $D = 0;
-        while ($R >= $L)
-        {
+        while ($R >= $L) {
             ($R == $L)?($D = 1):($D = 0);
             push @$tiles, Games::Domino::Tile->new({ left => $L, right => $R, double => $D });
             $L++;
@@ -358,25 +403,12 @@ sub _prepare
     return $tiles;
 }
 
-sub _open_ends
-{
-    my $self = shift;
-    my $ends = shift;
-    return if (defined($self->{open_ends}) && (scalar(@{$self->{open_ends}}) == 0));
-
-    push @{$self->{open_ends}}, @{$ends};
-}
-
-sub _next
-{
+sub _next {
     my $self = shift;
 
-    if ($self->{current}->name eq 'H')
-    {
+    if ($self->{current}->name eq 'H') {
         $self->{current} = $self->{computer};
-    }
-    else
-    {
+    } else {
         $self->{current} = $self->{human};
     }
 }
